package com.example.cxfrs;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.jaxrs.json.JacksonJaxbJsonProvider;
import com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class JacksonJaxbProviderFactory {
    /**
     * The default date format used for configuring {@link ObjectMapper} instances.
     */
    public static final String YYYY_MM_DD_FORMAT = "yyyy-MM-dd";

    /**
     * The default value to be configured for {@link DeserializationFeature}{@code .FAIL_ON_UNKNOWN_PROPERTIES}.
     */
    private static final boolean FAIL_ON_UNKNOWN_PROPERTIES = false;

    /**
     * The default value to be configured for {@link SerializationFeature}{@code .FAIL_ON_EMPTY_BEANS}.
     */
    private static final boolean FAIL_ON_EMPTY_BEANS = false;

    /**
     * The default value to be configured for {@link SerializationFeature}{@code .WRITE_DATES_AS_TIMESTAMPS}.
     */
    private static final boolean WRITE_DATES_AS_TIMESTAMPS = false;

    /**
     * The default value to be configured for {@link JsonInclude.Include} strategy.
     */
    private static final JsonInclude.Include DEFAULT_INCLUDE_NON_EMPTY = JsonInclude.Include.NON_EMPTY;

    /**
     * Non-public constructor
     */
    protected JacksonJaxbProviderFactory() {
    }

    /**
     * Creates an instance of {@link JacksonJsonProvider} that do NOT unwrap the JSON payload
     * on deserialization, do NOT wrap payload on serialization, configured out of the box
     * {@link JsonInclude.Include}{@code .NON_EMPTY} and {@code 'yyyy-MM-dd'} date format.
     *
     * @return Instance of {@link JacksonJsonProvider}.
     *
     * @see ObjectMapper
     * @see DeserializationFeature
     * @see SerializationFeature
     * @see JsonInclude.Include
     */
    public static JacksonJsonProvider newInstance() {
        final Map<DeserializationFeature, Boolean> deserialization = new HashMap<DeserializationFeature, Boolean>();
        deserialization.put(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, FAIL_ON_UNKNOWN_PROPERTIES);

        final Map<SerializationFeature, Boolean> serialization = new HashMap<SerializationFeature, Boolean>();
        serialization.put(SerializationFeature.FAIL_ON_EMPTY_BEANS, FAIL_ON_EMPTY_BEANS);
        serialization.put(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, WRITE_DATES_AS_TIMESTAMPS);

        return create(deserialization, serialization, DEFAULT_INCLUDE_NON_EMPTY, new SimpleDateFormat(YYYY_MM_DD_FORMAT));
    }

    /**
     * Creates a new instance of {@link JacksonJsonProvider} with a fine tuned configuration options and ObjectMapper settings.
     *
     * @param deserialization {@link Map} containing {@link DeserializationFeature} as the key and boolean flag to enable/disable as the value.
     * @param serialization {@link Map} containing {@link SerializationFeature} as the key and boolean flag to enable/disable as the value.
     * @param serializationInclusion {@link JsonInclude.Include} strategy annotation to be included to {@link ObjectMapper} as part of serialization.
     * @param dateFormat the {@link DateFormat} to be configured on {@link ObjectMapper}.
     *
     * @return Instance of {@link JacksonJsonProvider} configured with the supplied parameters.
     *
     * @see DeserializationFeature
     * @see SerializationFeature
     * @see JsonInclude.Include
     * @see DateFormat
     */
    public static JacksonJsonProvider create(final Map<DeserializationFeature, Boolean> deserialization, final Map<SerializationFeature, Boolean> serialization, final JsonInclude.Include serializationInclusion, final DateFormat dateFormat) {
        final JacksonJsonProvider provider = new JacksonJaxbJsonProvider();

        if (deserialization != null && !deserialization.isEmpty()) {
            deserialization.forEach(provider::configure);
        }

        if (serialization != null && !serialization.isEmpty()) {
            serialization.forEach(provider::configure);
        }

        // retrieve ObjectMapper from the provider
        final ObjectMapper mapper = Objects.requireNonNull(
                provider.locateMapper(Response.class, MediaType.APPLICATION_JSON_TYPE),
                ObjectMapper.class.getName() + "cannot be retrieved from the JacksonJsonProvider");

        if (serializationInclusion != null) {
            mapper.setSerializationInclusion(serializationInclusion);
        }

        if (dateFormat != null) {
            mapper.setDateFormat(dateFormat);
        }

        // support case-insensitive enum deserialization
        mapper.enable(MapperFeature.ACCEPT_CASE_INSENSITIVE_ENUMS);

        return provider;
    }
}
